"""
Simple Python's logging formater which formats :class:`logging.LogRecord`
to *JSON* object.
"""

import logging
import os
import re

import ujson

__all__ = ['JsonFormatter']


def _get_version():
    filename = os.path.join(os.path.dirname(__file__), 'CHANGELOG.md')
    with open(filename, 'rt') as fd:
        pat = r"""
            (?P<version>\d+\.\d+)         # minimum 'N.N'
            (?P<extraversion>(?:\.\d+)*)  # any number of extra '.N' segments
            (?:
                (?P<prerel>[abc]|rc)      # 'a' = alpha, 'b' = beta
                                          # 'c' or 'rc' = release candidate
                (?P<prerelversion>\d+(?:\.\d+)*)
            )?
            (?P<postdev>(\.post(?P<post>\d+))?(\.dev(?P<dev>\d+))?)?
        """
        for line in fd:
            match = re.search(pat, line, re.VERBOSE)
            if match:
                return match.group()
    raise ValueError("Can't get version")


__version__ = _get_version()


class JsonFormatter(logging.Formatter):
    """
    Python's logging formater which formats :class:`logging.LogRecord`
    to JSON object. If *pretty* is :data:`True`, attribute names will be
    sorted and indented. *ignored_attrs* as a :class:`list` contains
    attributes which will be excluded (for attribute names see
    https://docs.python.org/3/library/logging.html#logrecord-attributes).
    *exc_info* and *stack_info* are always excluded, they are converted
    to *exc_text* and *stack_text*.
    """

    def __init__(
            self, datefmt=None, style='%', pretty=False, ignored_attrs=None):
        super().__init__(datefmt=datefmt, style=style)
        if pretty:
            self.sort_keys = True
            self.indent = 4
        else:
            self.sort_keys = False
            self.indent = 0
        if ignored_attrs is not None:
            self.ignored_attrs = frozenset(ignored_attrs)
        else:
            self.ignored_attrs = frozenset()

    def format(self, record):
        record.message = record.getMessage()
        record.asctime = self.formatTime(record, self.datefmt)

        result_dict = {
            k: v for k, v in vars(record).items()
            if (not k.startswith('_') and
                k not in ['exc_info', 'stack_info'] and
                k not in self.ignored_attrs)
        }
        if 'exc_text' not in self.ignored_attrs:
            if record.exc_info:
                if not record.exc_text:
                    record.exc_text = self.formatException(record.exc_info)
                result_dict['exc_text'] = record.exc_text
            else:
                result_dict['exc_text'] = ''
        if 'stack_text' not in self.ignored_attrs:
            if record.stack_info:
                result_dict['stack_text'] = self.formatStack(record.stack_info)
            else:
                result_dict['stack_text'] = ''

        return ujson.dumps(
            result_dict, indent=self.indent, sort_keys=self.sort_keys)
